# twbot.rb: Twitter Bot Support Library
# Version 0.12
# (C)2008 H.Hiro(Maraigue) http://hhiro.net/about/
# 
# Twitterのbot、特に「定期的にデータを取得→定期的に取得したデータを投稿」
# するタイプのbotの作成を支援するライブラリです。
# 
# 複製・改変の条件は（新）BSDライセンスに合わせます。
# 末尾の著作権表示をご覧下さい。

require 'net/http'
require 'yaml'
require 'rexml/document'
require 'cgi'

$KCODE = 'u'

class TwBot
  # 継承先にて、データを取得するためのコードを記述する。
  # 返り値は文字列の配列でなければならない。
  def load_data
    raise NotImplementedError, "Method 'load_data' must be implemented"
  end
  private :load_data
  
  # 発言の投稿を行う
  def post_data
    # 投稿する発言が残っていない場合
    if @config[@list].empty?
      @logmsg = "(No post remain)"
      return
    end
    
    # 投稿する
    post = @config[@list].first.to_s
    if post.empty?
      @logmsg = "(Empty string specified. Posting canceled.)"
      @config[@list].shift
      return
    end
    if post == @config["last_post/"]
    	post += " ."
    end
    
    res = nil
    
    http = Net::HTTP.new('twitter.com')
    unless @testmode
      http.start do |w|
        req = Net::HTTP::Post.new('/statuses/update.xml')
        req.basic_auth(@config['login/'], @config['password/'])
        res = w.request(req, 'source=twbotrb&status='+CGI.escape(post))
      end
    else
      res = Struct.new(:body).new('<status></status>')
    end
    
    # 正常に投稿されたか確認する
    # （正常に投稿されたとき…正しいXMLであり、かつ"<status>"が含まれている）
    unless res.body.index("<status>")
      raise RuntimeError, "Posting a tweet has failed"
    end
    xml = REXML::Document.new(res.body)
    
    # 結果とログを出力
    STDERR.puts "[Updated!#{@testmode ? '(test)' : ''}] #{post}"
    @config[@list].shift
    @config["last_post/"] = post
    @logmsg = "(A tweet has posted)"
  end
  private :post_data
  
  # コンストラクタ
  # mode:
  #   動作("load"あるいは"post")を指定します。
  # config_file:
  #   設定ファイルです。YAML形式で保存されます。
  #   以下のキーを含むハッシュでなければなりません。
  #   "login/":ログイン名
  #   "password/":パスワード
  # log_file:（省略可能）
  #   ログを出力するファイルです。
  #   指定しないかnilを指定すると、ログのファイルへの出力を行いません。
  # list_name:（省略可能）
  #   1つの設定ファイルには複数のリストを持たせることが出来ます。
  #   mode=="load"の場合、ここで指定されたリストに発言の内容が格納されます。
  #   mode=="post"の場合、ここで指定されたリストの発言のうち先頭にある発言が
  #   投稿されます。
  # keep_config:（省略可能）
  #   trueの場合、設定ファイルの更新を行いません。
  #   なおfalseが指定されても、処理に失敗した場合（発言取得および投稿において
  #   例外が発生した場合）は設定ファイルの更新は行いません。
  # testmode: （省略可能、mode == "post"のときのみ有効)
  #   trueの場合、実際の発言の投稿を行いません。
  def initialize(mode, config_file, log_file = nil, list_name = '',
                 keep_config = false, testmode = false)
    @list = "data/#{list_name}"
    
    @config = YAML.load_file(config_file)
    @config['login/'] ||= ''
    @config['password/'] ||= ''
    @config[@list] ||= []
    
    @log = log_file ? open(log_file, 'a') : nil
    
    @logmsg = '(No error)' # この値がログファイルに書き込まれる
    @write_config = !keep_config
    @testmode = testmode
    
    # modeに応じて処理
    outmsg = nil
    begin
      case mode
      when 'load'
        @config[@list].concat(load_data())
      when 'post'
        post_data
      else
        raise ArgumentError, 'Not a valid mode'
      end
    rescue Exception => e
      outmsg = "[#{Time.now}]<#{mode}> #{e}: #{e.backtrace.join(%'\n')}"
      @write_config = false
    else
      outmsg = "[#{Time.now}]<#{mode}> #{@logmsg}"
    end
    STDERR.puts outmsg
    @log.puts outmsg if @log
    @log.close if @log
    
    # 設定ファイルに書き込む
    if @write_config
      # YAML.dumpでこける場合に備え、openの前にdumpしている
      buf = YAML.dump(@config)
      open(config_file, 'w'){ |file| file.print buf }
    end
  end
end

# Copyright (c) 2008, H.Hiro(Maraigue)
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
# Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
# Neither the name of the Maraigue nor the names of its contributors
# may be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
